<?php

namespace Drupal\php_password\Password;

use Drupal\Core\Password\PasswordInterface;

/**
 * Secure password hashing functions based on PHP password hashing functions.
 *
 * @see http://php.net/manual/en/ref.password.php
 */
class PhpPassword2 implements PasswordInterface {

  /**
   * The algorithm constant used to hash password.
   *
   * @see password_hash().
   * @see http://php.net/manual/en/password.constants.php
   */
  protected string $algorithm;

  /**
   * Password hash options for the provided algorithm.
   */
  protected array $options = [];

  /**
   * Constructs a new password hashing instance.
   *
   * @param string $algorithm
   *   The hashing algorithm to use. Defaults to php default.
   * @param array $options
   *   List of options. Refer to password_hash for available options.
   */
  public function __construct(string $algorithm = PASSWORD_DEFAULT, array $options = []) {
    $this->algorithm = $algorithm;
    $this->options = $options;
  }

  /**
   * {@inheritdoc}
   */
  public function hash($password): bool|string {
    // Prevent DoS attacks by refusing to hash large passwords.
    if (strlen($password) > static::PASSWORD_MAX_LENGTH) {
      return FALSE;
    }

    return password_hash($password, $this->algorithm, $this->options);
  }

  /**
   * {@inheritdoc}
   */
  public function check($password, $hash): bool {
    return password_verify($password, $hash);
  }

  /**
   * {@inheritdoc}
   */
  public function needsRehash($hash): bool {
    // The PHP password_needs_rehash() will return TRUE in two cases:
    // - The password is a Drupal 6 or 7 password, and it has been rehashed
    //   during the migration. In this case the rehashed legacy hash is prefixed
    //   to indicate an old Drupal hash and will not comply with the expected
    //   password_needs_rehash() format.
    // - The parameters of hashing engine were changed. For example the
    //   parameter 'password_hash_cost' (the hashing cost) has been increased in
    //   core.services.yml.
    return password_needs_rehash($hash, $this->algorithm, $this->options);
  }

}
