<?php

namespace Drupal\Tests\php_password\Unit\Password;

use Drupal\Core\Password\PasswordInterface;
use Drupal\php_password\Password\Drupal8Password;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;

/**
 * Test Drupal8Password proxy class.
 *
 * @coversDefaultClass \Drupal\php_password\Password\Drupal8Password
 * @group php_password
 */
class Drupal8PasswordTest extends UnitTestCase {


  /**
   * Mock PHP Password service being tested.
   *
   * @var \Drupal\Core\Password\PasswordInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $phpPassword;

  /**
   * Mock legacy password service being tested.
   *
   * @var \Drupal\Core\Password\PasswordInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $legacyPassword;

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->phpPassword = $this->prophesize(PasswordInterface::class);
    $this->legacyPassword = $this->prophesize(PasswordInterface::class);
  }

  /**
   * Test that we only ever call php_password implementation.
   *
   * @covers ::hash
   * @dataProvider providePasswordHashes
   */
  public function testHash($password) {
    $testPassword = new Drupal8Password(
      $this->phpPassword->reveal(),
      $this->legacyPassword->reveal()
    );
    $this->phpPassword->hash($password)
      ->shouldBeCalledTimes(1)
      ->willReturn('my hash');
    $this->legacyPassword->hash(Argument::any())
      ->shouldNotBeCalled();
    $this->assertEquals('my hash', $testPassword->hash($password));
  }

  /**
   * Test that check delegates to the correct implementation.
   *
   * @covers ::check
   * @dataProvider providePasswordHashes
   */
  public function testCheck(
    $password,
    $hash,
    $actual_pass,
    $actual_hash,
    $legacy
  ) {
    $testPassword = new Drupal8Password(
      $this->phpPassword->reveal(),
      $this->legacyPassword->reveal()
    );
    if ($legacy) {
      // This isn't working as expected... Somehow the comparison always fails.
      $this->legacyPassword->check($actual_pass, Argument::Any())
        ->shouldBeCalledTimes(1)
        ->willReturn(TRUE);
      $this->phpPassword->check(Argument::any(), Argument::any())
        ->shouldNotBeCalled();
    }
    else {
      $this->phpPassword->check($actual_pass, $actual_hash)
        ->shouldBeCalledTimes(1)
        ->willReturn(TRUE);
      $this->legacyPassword->check(Argument::any(), Argument::any())
        ->shouldNotBeCalled();
    }
    $testPassword->check($password, $hash);
  }

  /**
   * Test that we only ever call php_password implementation.
   *
   * @covers ::needsRehash
   * @dataProvider providePasswordHashes
   */
  public function testNeedsRehash($password, $hash): void {
    $testPassword = new Drupal8Password(
      $this->phpPassword->reveal(),
      $this->legacyPassword->reveal()
    );
    $this->phpPassword->needsRehash($hash)
      ->shouldBeCalledTimes(1)
      ->willReturn(TRUE);
    $this->legacyPassword->needsRehash(Argument::any())
      ->shouldNotBeCalled();
    $this->assertTrue($testPassword->needsRehash($hash));
  }

  /**
   * Provide php password hashing fixtures.
   */
  public function providePasswordHashes(): array {
    return [
      'rehashed d6 md5 S' => [
        'password',
        'U$S$asdf',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$S$asfd',
        TRUE,
      ],
      'rehashed d6 md5 H' => [
        'password',
        'U$H$asdf',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$H$asfd',
        TRUE,
      ],
      'rehashed d6 md5 P' => [
        'password',
        'U$P$asdf',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$P$asfd',
        TRUE,
      ],
      'rehashed d6 md5 2' => [
        'password',
        'U$2$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$2$asfd',
        FALSE,
      ],
      'rehashed d6 md5 2a' => [
        'password',
        'U$2a$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$2a$asfd',
        FALSE,
      ],
      'rehashed d6 md5 2b' => [
        'password',
        'U$2b$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$2b$asfd',
        FALSE,
      ],
      'rehashed d6 md5 2x' => [
        'password',
        'U$2x$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$2x$asfd',
        FALSE,
      ],
      'rehashed d6 md5 2y' => [
        'password',
        'U$2y$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$2y$asfd',
        FALSE,
      ],
      'rehashed d6 md5 argon2i' => [
        'password',
        'U$argon2i$asfd',
        '5f4dcc3b5aa765d61d8327deb882cf99',
        '$argon2i$asfd',
        FALSE,
      ],

      // Normally hashed passwords.
      ['password', '$S$asdf', 'password', '$S$asfd', TRUE],
      ['password', '$H$asdf', 'password', '$H$asfd', TRUE],
      ['password', '$P$asdf', 'password', '$P$asfd', TRUE],
      ['password', '$2$asfd', 'password', '$2$asfd', FALSE],
      ['password', '$2a$asfd', 'password', '$2a$asfd', FALSE],
      ['password', '$2b$asfd', 'password', '$2b$asfd', FALSE],
      ['password', '$2x$asfd', 'password', '$2x$asfd', FALSE],
      ['password', '$2y$asfd', 'password', '$2y$asfd', FALSE],
      ['password', '$argon2i$asfd', 'password', '$argon2i$asfd', FALSE],
      // ['password', 'hash', 'actualpass', 'actualhash',],
    ];
  }

}
